﻿namespace TabsStudioSQLComment
{
    class CommandsImpl
    {
        public CommandsImpl(TabManager tabManager_, TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            tabManager = tabManager_;
            engine = engine_;
            dte = dte_;
        }

        public void CommentTabExtension(TabsStudioExt.Tab tab, TabsStudioExt.TabExtension ext)
        {
            CommentTabExtension(tab, ext, EnterComment(ext));
        }

        public void CommentFromSelectionTabExtension(TabsStudioExt.Tab tab, TabsStudioExt.TabExtension ext)
        {
            CommentTabExtension(tab, ext, GetSelectionText());
        }

        public void CommentTab(TabsStudioExt.Tab tab)
        {
            TabsStudioExt.TabExtension ext = GetActiveExtension(tab);
            string newComment = EnterComment(ext);
            if (newComment == null)
                return;

            tabManager.SetTabExtensionComment(ext, newComment);
        }

        public void CommentFromSelectionTab(TabsStudioExt.Tab tab)
        {
            TabsStudioExt.TabExtension ext = GetActiveExtension(tab);
            string newComment = GetSelectionText();
            if (newComment == null)
                return;

            tabManager.SetTabExtensionComment(ext, newComment);
        }

        public void SetSQLComment()
        {
            TabsStudioExt.Tab tab = FindSelectedTab();
            if (tab != null)
                CommentTabExtension(tab, GetActiveExtension(tab));
        }

        public void SetSQLCommentFromSelection()
        {
            TabsStudioExt.Tab tab = FindSelectedTab();
            if (tab != null)
                CommentTabExtension(tab, GetActiveExtension(tab), GetSelectionText());
        }

        private string GetSelectionText()
        {
            if (dte.ActiveDocument == null)
                return null;

            EnvDTE.TextSelection ts = dte.ActiveDocument.Selection as EnvDTE.TextSelection;
            if (ts == null || ts.Text == null || ts.Text.Length == 0)
                return null;

            return ts.Text;
        }

        private void CommentTabExtension(TabsStudioExt.Tab tab, TabsStudioExt.TabExtension ext, string newComment)
        {
            if (newComment == null)
                return;

            if (tab.TabExtensions.Count == 1)
                tabManager.SetTabComment(tab, newComment);
            else
                tabManager.SetTabExtensionComment(ext, newComment);
        }

        private TabsStudioExt.Tabs FindSelectedTabs()
        {
            foreach (TabsStudioExt.Tabs tabs in engine.GetTabs())
            {
                if (tabs.IsGroupSelected)
                    return tabs;
            }
            return null;
        }

        private TabsStudioExt.Tab FindSelectedTab()
        {
            TabsStudioExt.Tabs tabs = FindSelectedTabs();
            if (tabs != null)
            {
                foreach (TabsStudioExt.Tab tab in tabs.TabList)
                {
                    if (tab.IsTabSelected)
                        return tab;
                }
            }
            return null;
        }

        private string EnterComment(TabsStudioExt.TabExtension ext)
        {
            CommentDialog dialog = new CommentDialog(ext.TitleManager.UntransformedTitle, tabManager.GetComment(ext.Window));
            System.Windows.Interop.WindowInteropHelper helper = new System.Windows.Interop.WindowInteropHelper(dialog);
            helper.Owner = new System.IntPtr(dte.MainWindow.HWnd);
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                return dialog.GetComment();
            else
                return null;
        }

        private static TabsStudioExt.TabExtension GetActiveExtension(TabsStudioExt.Tab tab)
        {
            foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
            {
                if (ext.IsExtensionActive)
                    return ext;
            }
            return null;
        }

        private TabManager tabManager;
        TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
    }
}
